<?php

namespace Drupal\private_message\Controller;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityManagerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\private_message\Ajax\PrivateMessageInsertNewMessagesCommand;
use Drupal\private_message\Ajax\PrivateMessageInsertThreadsCommand;
use Drupal\private_message\Ajax\PrivateMessageInsertPreviousMessagesCommand;
use Drupal\private_message\Ajax\PrivateMessageMembersAutocompleteResponseCommand;
use Drupal\private_message\Service\PrivateMessageServiceInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

class AjaxController extends ControllerBase implements AjaxControllerInterface  {

  const AUTOCOMPLETE_COUNT = 10;

  /**
   * The renderer service
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The request stack
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The entity manager service
   *
   * @param \Drupal\Core\Entity\EntityManagerInterface
   */
  protected $entityManager;

  /**
   * The private message service
   *
   * @var \Drupal\private_message\Service\PrivateMessageServiceInterface
   */
  protected $privateMessageService;

  /**
   * Constructs n AjaxController object
   *
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   The request stack
   * @param \Drupal\Core\Entity\EntityManagerInterface $entityManager
   *   The entity manager service
   * @param \Drupal\private_message\Service\PrivateMessageServiceInterface $privateMessageService
   *   The private message service
   */
  public function __construct(RendererInterface $renderer, RequestStack $requestStack, EntityManagerInterface $entityManager, PrivateMessageServiceInterface $privateMessageService) {
    $this->renderer = $renderer;
    $this->requestStack = $requestStack;
    $this->entityManager = $entityManager;
    $this->privateMessageService = $privateMessageService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('renderer'),
      $container->get('request_stack'),
      $container->get('entity.manager'),
      $container->get('private_message.service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function ajaxCallback($op) {
    $response = new AjaxResponse();

    switch($op) {
      case 'get_new_messages': {
        $thread_id = $this->requestStack->getCurrentRequest()->get('threadid');
        $message_id = $this->requestStack->getCurrentRequest()->get('messageid');
        if (is_numeric($thread_id) && is_numeric($message_id)) {
          $new_messages = $this->privateMessageService->getNewMessages($thread_id, $message_id);
          if (count($new_messages)) {
            $messages = [];
            $view_builder = $this->entityManager->getViewBuilder('private_message');
            foreach ($new_messages as $message) {
              $messages[] = $view_builder->view($message);
            }

            $command = new PrivateMessageInsertNewMessagesCommand($this->renderer->renderRoot($messages));
            $response->addCommand($command);
          }
        }

        break;
      }

      case 'get_old_messages': {
        $current_request = $this->requestStack->getCurrentRequest();
        $thread_id = $current_request->get('threadid');
        $message_id = $current_request->get('messageid');
        $message_count = $current_request->get('count');
        if (is_numeric($thread_id) && is_numeric($message_id)) {
          $message_info = $this->privateMessageService->getPreviousMessages($thread_id, $message_id);

          if (count($message_info)) {
            $messages = [];
            $view_builder = $this->entityManager->getViewBuilder('private_message');
            foreach ($message_info as $message) {
              $messages[] = $view_builder->view($message);
            }

            $command = new PrivateMessageInsertPreviousMessagesCommand($this->renderer->renderRoot($messages));
          }
          else {
            $command = new PrivateMessageInsertPreviousMessagesCommand('');
          }

          $response->addCommand($command);
        }

        break;
      }

      case 'get_old_inbox_threads': {
        $timestamp = $this->requestStack->getCurrentRequest()->get('timestamp');
        $thread_count = $this->requestStack->getCurrentRequest()->get('count');
        if (is_numeric($timestamp)) {
          $thread_info = $this->privateMessageService->getThreadsForUser($thread_count, $timestamp);
          if (count($thread_info['threads'])) {
            $oldest_timestamp = FALSE;
            $view_builder = $this->entityManager->getViewBuilder('private_message_thread');
            $threads = [];
            foreach ($thread_info['threads'] as $thread) {
              if ($thread_info['next_exists']) {
                $oldest_timestamp = $thread->get('updated')->value;
              }
              $threads[] = $view_builder->view($thread, 'inbox');
            }

            $command = new PrivateMessageInsertThreadsCommand($this->renderer->renderRoot($threads), $oldest_timestamp);
          }
          else {
            $command = new PrivateMessageInsertThreadsCommand('', FALSE);
          }

          $response->addCommand($command);
        }

        break;
      }
    }

    return $response;
  }

  public function privateMessageMembersAutocomplete() {
    $response = new AjaxResponse();

    $username = $this->requestStack->getCurrentRequest()->get('username');
    $accounts = $this->privateMessageService->getUsersFromString($username, self::AUTOCOMPLETE_COUNT);
    $user_info = [];
    foreach ($accounts as $account) {
      $user_info[] = [
	      'uid' => $account->id(),
	      'username' => $account->getDisplayName(),
      ];
    }

    $response->addCommand(new PrivateMessageMembersAutocompleteResponseCommand($username, $user_info));

    return $response;
  }
}
