<?php

namespace Drupal\private_message\Service;

use Drupal\user\UserInterface;

interface PrivateMessageServiceInterface {

  /**
   * Retrieves the private message thread for the
   * given members. If no thread exists, one will be created.
   *
   * @param Drupal\user\UserInterface[] $members
   *   An array of User objects for whom the private message
   *   thread should be retrieved.
   *
   * @return Drupal\private_message\Entity\PrivateMessageThread
   */
  public function getThreadForMembers(array $members);

  /**
   * Get the most recently updated thread for the given user
   *
   * @param \Drupal\user\Entity\UserInterface $user
   *   The user whose most recently updated thread should be retrieved
   */
  public function getFirstThreadForUser(UserInterface $user);

  /**
   * Retrieve private message threads for a given user
   *
   * @param int $count
   *   The number of threads to retrieve
   * @param int $timestamp
   *    A timestamp relative to which only threads with an earlier timestamp should be returned
   *
   * @return array
   *   An array with two keys:
   *   - threads: an array of \Drupal\private_message\Entity\PrivateMessageThread
   *   - next_exists: a boolean indicating whether any more private message threads exist after the last one
   */
  public function getThreadsForUser($count, $timestamp = FALSE);
  /**
   * Retrieve new private messages for a user that have been created after
   * the given ID
   *
   * @param int $thread_id
   *   The ID of the thread from which messages should be retrieved
   * @param int $message_id
   *   The ID after which messages should be retrieved
   *
   * @return array
   *   An array containing any new messages that the user has permission to view
   */
  public function getNewMessages($thread_id, $message_id);

  /**
   * Retrieve old messages for a user that were created before the given ID
   *
   * @param int $thread_id
   *   The ID of the thread from which messages should be retrieved
   * @param int $message_id
   *   The ID before which messages should be retrieved
   *
   * @return array
   *   An array containing the following to keys:
   *   - messages: an array of \Drupal\private_message\Entity\PrivateMessage
   *   - next_exists: a boolean indicating whether or not any messages appear
   *     in the thread before the first message returned
   */
  public function getPreviousMessages($thread_id, $message_id);

  /**
   * Get a list of User objects whose account names begin with the given string.
   * Only accounts that have 'Use private messaging system' permission will be
   * returned, and the viewing user must have both 'View user information' and
   * 'access user profiles' to get any results at all.
   *
   * @param string $string
   *   The string to search for.
   * @param int $count
   *   The maximum number of results to return
   *
   * @return \Drupal\user\Entity\User[]
   *   An array of User accounts whose account names begin with the given string.
   */
  public function getUsersFromString($string, $count);
}
